require 'test_helper'

class PuppetclassTest < ActiveSupport::TestCase
  setup do
    User.current = User.find_by_login "admin"
  end

  test "name can't be blank" do
    puppet_class = Puppetclass.new
    assert !puppet_class.save
  end

  test "name can't contain trailing white spaces" do
    puppet_class = Puppetclass.new :name => "   test     class   "
    assert !puppet_class.name.strip.squeeze(" ").empty?
    assert !puppet_class.save

    puppet_class.name.strip!.squeeze!(" ")
    assert puppet_class.save
  end

  test "name must be unique" do
    puppet_class = Puppetclass.new :name => "test class"
    assert puppet_class.save

    other_puppet_class = Puppetclass.new :name => "test class"
    assert !other_puppet_class.save
  end

  test "scanForClasses should retrieve puppetclasses from .pp files" do
    path = "/some/path"
    puppet_classes = ["class some_puppet_class {","class other_puppet_class{","class yet_another_puppet_class{"]
    mock(Dir).glob("#{path}/*/manifests/**/*.pp") { puppet_classes }
    puppet_classes.each do |puppet_class|
      mock(File).read(anything) { StringIO.new(puppet_class) }
    end

    klasses = Puppetclass.scanForClasses path
    assert klasses[0] == "some_puppet_class"
    assert klasses[1] == "other_puppet_class"
    assert klasses[2] == "yet_another_puppet_class"
  end

  def setup_user operation
    @one = users(:one)
    as_admin do
      role = Role.find_or_create_by_name :name => "#{operation}_puppetclasses"
      role.permissions = ["#{operation}_puppetclasses".to_sym]
      @one.roles = [role]
      @one.save!
    end
    User.current = @one
  end

  test "user with create permissions should be able to create" do
    setup_user "create"
    record =  Puppetclass.create :name => "dummy"
    assert record.valid?
    assert !record.new_record?
  end

  test "user with view permissions should not be able to create" do
    setup_user "view"
    record =  Puppetclass.create :name => "dummy"
    assert record.valid?
    assert record.new_record?
  end

  test "user with destroy permissions should be able to destroy" do
    setup_user "destroy"
    record =  Puppetclass.first
    as_admin do
      record.hosts = []
    end
    assert record.destroy
    assert record.frozen?
  end

  test "user with edit permissions should not be able to destroy" do
    setup_user "edit"
    record =  Puppetclass.first
    assert !record.destroy
    assert !record.frozen?
  end

  test "user with edit permissions should be able to edit" do
    setup_user "edit"
    record      =  Puppetclass.first
    record.name = "renamed"
    assert record.save
  end

  test "user with destroy permissions should not be able to edit" do
    setup_user "destroy"
    record      =  Puppetclass.first
    record.name = "renamed"
    as_admin do
      record.hosts = []
    end
    assert !record.save
    assert record.valid?
  end

end
